from collections.abc import Awaitable
import logging
from .base_tool import Tool
from typing import Any
from contextlib import AsyncExitStack
from mcp import ClientSession, StdioServerParameters, McpError
from mcp.client.stdio import stdio_client
from mcp.client.sse import sse_client
from volcenginesdkarkruntime.types.chat import ChatCompletionToolParam

logger = logging.getLogger(__name__)


class MCPClient:
    def __init__(self) -> None:
        self.session: ClientSession | None = None
        self._server_params: dict[str, Any] | None = None
        self.exit_stack = AsyncExitStack()

    async def connect_to_server(self, params: dict[str, Any] | None) -> None:
        if not params:
            return
        if "url" in params:
            await self.connect_to_sse_server(**params)
        elif "command" in params:
            await self.connect_to_stdio_server(**params)
        else:
            raise ValueError(f"Invalid parameters: {params}")

    async def connect_to_stdio_server(
        self,
        command: str,
        args: list[str],
        env: dict[str, str] | None = None,
        cwd: str | None = None,
        encoding: str = "utf-8",
    ):
        """Connect to an MCP server

        Args:
            _server_params: parameters for the server
        """

        self._server_params = {
            "command": command,
            "args": args,
            "env": env,
            "cwd": cwd,
            "encoding": encoding,
        }
        _server_params = StdioServerParameters(
            command=command, args=args, env=env, cwd=cwd, encoding=encoding
        )
        stdio_transport = await self.exit_stack.enter_async_context(
            stdio_client(_server_params)
        )
        self.read, self.write = stdio_transport
        self.session = await self.exit_stack.enter_async_context(
            ClientSession(self.read, self.write)
        )

        if not self.session:
            raise ValueError("Failed to create session")

        await self.session.initialize()

        # List available tools
        response = await self.session.list_tools()
        tools = response.tools
        self.tool_list = [tool.name for tool in tools]
        logger.info(
            f"Successfully connected to MCP server with tools: {self.tool_list}"
        )

    async def connect_to_sse_server(
        self,
        url: str,
        headers: dict[str, str] | None = None,
        timeout: float = 5,
        sse_read_timeout: float = 60 * 5,
    ):
        """Connect to an MCP server

        Args:
            url: URL of the server
            headers: headers to send with the request
        """
        self._server_params = {
            "url": url,
            "headers": headers,
            "timeout": timeout,
            "sse_read_timeout": sse_read_timeout,
        }
        sse_transport = await self.exit_stack.enter_async_context(
            sse_client(
                url=url,
                headers=headers,
                timeout=timeout,
                sse_read_timeout=sse_read_timeout,
            )
        )
        self.read, self.write = sse_transport
        self.session = await self.exit_stack.enter_async_context(
            ClientSession(self.read, self.write)
        )

        if not self.session:
            raise ValueError("Failed to create session")

        await self.session.initialize()

        response = await self.session.list_tools()
        tools = response.tools
        self.tool_list = [tool.name for tool in tools]
        logger.info(
            f"Successfully connected to MCP server with tools: {self.tool_list}"
        )

    @property
    def available_tools(self) -> list["MCPTool"]:
        """Get a list of available tools"""
        return [MCPTool(tool, self) for tool in self.tool_list]

    async def reconnect(self):
        """Reconnect to the server"""
        logger.info(f"Reconnecting to MCP server: {self._server_params}")
        if not self.session:
            raise ValueError("MCPClient not connected yet")
        await self.cleanup()
        await self.connect_to_server(self._server_params)

    async def cleanup(self):
        """Clean up resources"""
        await self.exit_stack.aclose()
        logger.info("MCPClient cleaned up")


class MCPTool(Tool):
    def __init__(self, name: str, client: MCPClient):
        self.name: str = name
        self.client: MCPClient = client
        self._initialized: bool = False

    async def init(self) -> None:
        """初始化工具信息，否则语言模型无法正确调用"""
        if not self.client.session:
            raise ValueError("MCPClient not connected")
        tool_list = (await self.client.session.list_tools()).tools
        for tool in tool_list:
            if tool.name == self.name:
                self.description: str = tool.description if tool.description else ""
                self.parameters: dict[str, Any] = tool.inputSchema
                self._initialized = True
                break
        else:
            raise ValueError(f"Tool {self.name} not found")

    def to_dict(self) -> ChatCompletionToolParam:
        if not self._initialized:
            raise ValueError(f"Tool '{self.name}' not initialized")
        return {
            "type": "function",
            "function": {
                "name": self.name,
                "description": self.description,
                "parameters": self.parameters,
            },
        }

    def __repr__(self) -> str:
        return (
            f"MCPTool("
            f"name={self.name!r}, "
            f"description={self.description!r}, "
            f"parameters={self.parameters!r})"
        )

    def __call__(self, *args, **kwargs) -> Awaitable[str]:
        return self._call(*args, **kwargs)

    async def _call(self, *args, **arguments) -> str:
        if args:
            raise ValueError(
                "MCPTool does not support positional arguments, try keyword arguments"
            )
        if not self.client.session:
            raise ValueError("MCPClient not connected")
        try:
            result = await self.client.session.call_tool(
                name=self.name, arguments=arguments
            )
        except McpError:
            try:
                logging.warning("MCP request failed, trying to reconnect to MCP server")
                await self.client.reconnect()
                result = await self.client.session.call_tool(
                    name=self.name, arguments=arguments
                )
            except McpError as e:
                logger.exception("MCP request failed after reconnecting")
                return f"MCP request failed after reconnecting: {e}"
        except Exception as e:
            logger.exception(f"Error when calling tool: {e}")
            return f"Error when calling tool: {e}"
        return str(result.content)


class FoldableMCPTool(MCPTool):
    foldable = True

    def __init__(self, name: str, client: MCPClient):
        super().__init__(name, client)
        self.description += (
            "This tool is foldable, which means the result of this tool "
            "will be hidden when a new round of conversation starts."
        )



from ..agent.function_tool import FunctionTool, FoldableFunctionTool


# 预设工具
@FoldableFunctionTool
def python_script_tool(script: str) -> str:
    """A tool that allows you to run python script and returns the full output. Tips: You can use `print` to check the result"""
    import subprocess

    result = subprocess.run(
        ["python", "-E", "-c", script], capture_output=True, text=True
    )
    return result.stdout + result.stderr


@FunctionTool
def get_time_tool() -> str:
    """A tool that can get the current time"""
    import time

    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())


@FunctionTool
def open_website_tool(url: str) -> str:
    """A tool that allows user to browse the website"""
    import webbrowser

    webbrowser.open(url)
    return f"Successfully opened {url}"


@FoldableFunctionTool
def browse_web_tool(url: str) -> str:
    """A tool that allows you to browse the website"""
    import httpx, bs4  # type: ignore

    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36"
    }
    response = httpx.get(url, headers=headers)
    soup = bs4.BeautifulSoup(response.text, "html.parser")
    return soup.get_text()



__author__ = "captainSuo"
__version__ = "0.1.7"
__all__ = [
    "Agent",
    "AsyncAgent",
    "AsyncFunctionTool",
    "Tool",
    "FoldableAsyncFunctionTool",
    "FoldableFunctionTool",
    "FoldableMCPTool",
    "FunctionTool",
    "MCPClient",
    "MCPTool",
]


from .agent.agent import Agent
from .agent.agent_async import AsyncAgent
from .agent.function_tool import (
    FunctionTool,
    FoldableFunctionTool,
    AsyncFunctionTool,
    FoldableAsyncFunctionTool,
)
from .agent.mcp_tool import MCPClient, MCPTool, FoldableMCPTool
from .agent.base_tool import Tool



from collections.abc import Mapping, Iterable
from typing_extensions import TypeAlias, NotRequired
from volcenginesdkarkruntime.types.chat import (
    ChatCompletionSystemMessageParam,
    ChatCompletionUserMessageParam,
    ChatCompletionAssistantMessageParam,
    ChatCompletionToolMessageParam,
    ChatCompletionFunctionMessageParam,
    ChatCompletionMessageToolCallParam,
    ChatCompletionContentPartParam,
)


JSONType: TypeAlias = (
    Mapping[str, "JSONType"] | list["JSONType"] | str | int | float | bool | None
)


class SystemMessageParam(ChatCompletionSystemMessageParam):
    time: NotRequired[int]


class UserMessageParam(ChatCompletionUserMessageParam):
    time: NotRequired[int]


class AssistantMessageParam(ChatCompletionAssistantMessageParam):
    time: NotRequired[int]


class ToolMessageParam(ChatCompletionToolMessageParam):
    time: NotRequired[int]


class FunctionMessageParam(ChatCompletionFunctionMessageParam):
    time: NotRequired[int]


MessageParam: TypeAlias = (
    SystemMessageParam
    | UserMessageParam
    | AssistantMessageParam
    | ToolMessageParam
    | FunctionMessageParam
)

ToolCallParam: TypeAlias = ChatCompletionMessageToolCallParam
ContentPartParam: TypeAlias = ChatCompletionContentPartParam
MessageContent: TypeAlias = Iterable[ContentPartParam] | str



from .function_tools import (
    python_script_tool,
    get_time_tool,
    open_website_tool,
    browse_web_tool,
)

__all__ = [
    "python_script_tool",
    "get_time_tool",
    "open_website_tool",
    "browse_web_tool",
]



import time
from typing import Self, cast, Literal
from collections.abc import Callable, Generator
from copy import deepcopy
from contextlib import contextmanager
from volcenginesdkarkruntime import Ark
from volcenginesdkarkruntime._streaming import Stream
from volcenginesdkarkruntime.types.chat.completion_create_params import Thinking
from volcenginesdkarkruntime.types.chat import ChatCompletion, ChatCompletionChunk

from ez_agent.types import (
    AssistantMessageParam,
    MessageContent,
    MessageParam,
    ToolCallParam,
    UserMessageParam,
)
from .base_tool import Tool


class Agent:

    def __init__(
        self: Self,
        model: str,
        api_key: str,
        base_url: str,
        instructions: str = "",
        tools: list[Tool] | None = None,
        frequency_penalty: float | None = None,
        temperature: float | None = None,
        top_p: float | None = None,
        max_tokens: int | None = None,
        max_completion_tokens: int | None = None,
        thinking: bool | None = None,
        message_expire_time: int | None = None,
    ) -> None:
        self._tools: dict[str, Tool] | None = {tool.name: tool for tool in tools} if tools else None
        self._client: Ark = Ark(api_key=api_key, base_url=base_url)
        self._api_key: str = api_key
        self._base_url: str = base_url

        self.model: str = model
        self.instructions: str = instructions
        self.messages: list[MessageParam] = [{"role": "system", "content": instructions}]
        self.response_handlers: list[Callable[[AssistantMessageParam], None]] = []
        self.stream_chunk_handlers: list[Callable[[str], None]] = []
        self.tool_call_handlers: list[Callable[[ToolCallParam], None]] = []
        self.reasoning_handlers: list[Callable[[str], None]] = []
        self.stream_reasoning_handlers: list[Callable[[str], None]] = []

        self.frequency_penalty: float | None = frequency_penalty
        self.temperature: float | None = temperature
        self.top_p: float | None = top_p
        self.max_tokens: int | None = max_tokens
        self.max_completion_tokens: int | None = max_completion_tokens
        self.thinking: bool | None = thinking

        self.message_expire_time: int | None = message_expire_time

    @property
    def client(self) -> Ark:
        return self._client

    @property
    def api_key(self) -> str:
        return self._api_key

    @property
    def base_url(self) -> str:
        return self._base_url

    @property
    def tools(self) -> list[Tool]:
        return list(self._tools.values()) if self._tools else []

    @tools.setter
    def tools(self, value: list[Tool] | None):
        self._tools = {tool.name: tool for tool in value} if value else None

    def get_tool(self, name: str) -> Tool | None:
        return self._tools.get(name) if self._tools else None

    def send_messages(self) -> AssistantMessageParam:
        thinking_mapping: dict[bool | None, Literal["enabled", "disabled", "auto"]] = {
            True: "enabled",
            False: "disabled",
            None: "auto",
        }
        thinking_param: Thinking = {"type": thinking_mapping.get(self.thinking) or "auto"}
        response: ChatCompletion | Stream[ChatCompletionChunk] = self.client.chat.completions.create(
            model=self.model,
            messages=self.messages,
            tools=([tool.to_dict() for tool in self._tools.values()] if self._tools else None),
            tool_choice="auto" if self._tools else "none",
            frequency_penalty=self.frequency_penalty,
            max_tokens=self.max_tokens,
            max_completion_tokens=self.max_completion_tokens,
            temperature=self.temperature,
            top_p=self.top_p,
            stream=False,
            thinking=thinking_param,
        )
        assert isinstance(response, ChatCompletion)
        result: AssistantMessageParam = cast(AssistantMessageParam, response.choices[0].message.to_dict())
        result["time"] = response.created
        reasoning_content = response.choices[0].message.reasoning_content
        if reasoning_content:
            for reasoning_handler in self.reasoning_handlers:
                reasoning_handler(reasoning_content)
        for response_handler in self.response_handlers:
            response_handler(result)
        return result

    def get_response(self) -> MessageContent | None:
        response: AssistantMessageParam = self.send_messages()
        tool_calls: list[ToolCallParam] | None = (
            cast(list[ToolCallParam], response.get("tool_calls")) if response.get("tool_calls") else None
        )
        self.messages.append(response)
        if tool_calls:
            self.call_tool(tool_calls)
            return self.get_response()
        return response.get("content")  # type: ignore

    def send_messages_stream(self) -> Generator[ChatCompletionChunk, None, None]:
        thinking_mapping: dict[bool | None, Literal["enabled", "disabled", "auto"]] = {
            True: "enabled",
            False: "disabled",
            None: "auto",
        }
        thinking_param: Thinking = {"type": thinking_mapping.get(self.thinking) or "auto"}
        response: ChatCompletion | Stream[ChatCompletionChunk] = self.client.chat.completions.create(
            model=self.model,
            messages=self.messages,
            tools=([tool.to_dict() for tool in self._tools.values()] if self._tools else None),
            tool_choice="auto" if self._tools else "none",
            frequency_penalty=self.frequency_penalty,
            max_tokens=self.max_tokens,
            max_completion_tokens=self.max_completion_tokens,
            temperature=self.temperature,
            top_p=self.top_p,
            stream=True,
            thinking=thinking_param,
        )
        assert isinstance(response, Stream)
        for chunk in response:
            if chunk.choices[0].finish_reason == "stop":
                break
            yield chunk

    def get_response_stream(self) -> MessageContent | None:
        response = self.send_messages_stream()
        collected_chunks: list[ChatCompletionChunk] = []
        collected_messages: list[str] = []
        collected_reasoning_messages: list[str] = []
        tool_calls_by_id: dict[int, ToolCallParam] = {}

        for chunk in response:
            collected_chunks.append(chunk)
            if chunk.choices[0].delta.content:
                collected_messages.append(chunk.choices[0].delta.content)
                for stream_chunk_handler in self.stream_chunk_handlers:
                    stream_chunk_handler(chunk.choices[0].delta.content)

            if chunk.choices[0].delta.reasoning_content:
                collected_reasoning_messages.append(chunk.choices[0].delta.reasoning_content)
                for resoning_stream_handler in self.stream_reasoning_handlers:
                    resoning_stream_handler(chunk.choices[0].delta.reasoning_content)

            # 处理工具调用
            if hasattr(chunk.choices[0].delta, "tool_calls") and chunk.choices[0].delta.tool_calls:
                for tool_call in chunk.choices[0].delta.tool_calls:
                    call_id = tool_call.index

                    if call_id not in tool_calls_by_id:
                        tool_calls_by_id[call_id] = {
                            "id": "",
                            "type": "function",
                            "function": {"name": "", "arguments": ""},
                        }

                    # 更新工具调用信息
                    current_tool = tool_calls_by_id[call_id]
                    if hasattr(tool_call, "function"):
                        if not tool_call.function:
                            continue
                        function_data = current_tool["function"]
                        if hasattr(tool_call.function, "name") and tool_call.function.name:
                            function_data["name"] = tool_call.function.name

                        if hasattr(tool_call.function, "arguments") and tool_call.function.arguments:
                            function_data["arguments"] += tool_call.function.arguments

                    if hasattr(tool_call, "id") and tool_call.id:
                        current_tool["id"] = tool_call.id

        # 转换工具调用字典为列表
        tool_calls: list[ToolCallParam] = []
        for tool_call in tool_calls_by_id.values():
            tool_calls.append(tool_call)

        full_content = "".join(collected_messages)
        message: AssistantMessageParam = {
            "role": "assistant",
            "content": full_content,
            "time": collected_chunks[-1].created,
        }

        if collected_reasoning_messages:
            reasoning_content: str = "".join(collected_reasoning_messages)
            for reasoning_handler in self.reasoning_handlers:
                reasoning_handler(reasoning_content)

        for response_handler in self.response_handlers:
            response_handler(message)

        if tool_calls:
            message["tool_calls"] = tool_calls
            self.messages.append(message)
            self.call_tool(tool_calls)

            return self.get_response_stream()
        else:
            self.messages.append(message)
            return message.get("content")  # type: ignore

    def call_tool(self, tool_calls: list[ToolCallParam]) -> None:
        # 因为模型会输出 ture/false 而不是 True/False，所以需要转换
        true: bool = True  # type: ignore
        false: bool = False  # type: ignore
        if not self._tools:
            return
        for tool_call in tool_calls:
            called_tool = self._tools[tool_call["function"]["name"]]
            result = str(called_tool(**eval(tool_call["function"]["arguments"])))

            self.messages.append(
                {
                    "role": "tool",
                    "content": result,
                    "tool_call_id": tool_call["id"],
                    "time": int(time.time()),
                }
            )
            for tool_call_handler in self.tool_call_handlers:
                tool_call_handler(tool_call)

    def _fold_previous_tool_results(self) -> None:
        if not self._tools:
            return
        for index, _message in enumerate(self.messages):
            if not _message.get("tool_calls"):
                continue
            for tool_call in _message["tool_calls"]:  # type: ignore
                tool_name: str = tool_call["function"]["name"]
                if not self._tools.get(tool_name):
                    continue
                if not self._tools[tool_name].foldable:
                    continue
                for i in range(index + 1, len(self.messages)):
                    if self.messages[i].get("role") != "tool":
                        continue
                    if self.messages[i].get("tool_call_id") == tool_call["id"]:
                        self.messages[i] = {
                            "role": "tool",
                            "content": "The result has been folded",
                            "tool_call_id": tool_call["id"],
                        }
                        break

    def run(
        self,
        content: MessageContent,
        user_name: str | None = None,
        stream: bool = False,
    ) -> str | None:
        if self.message_expire_time:
            self.clear_msg_by_time(self.message_expire_time)
        self._fold_previous_tool_results()
        import time

        user_message: UserMessageParam = {
            "role": "user",
            "content": content,
            "time": int(time.time()),
        }
        if user_name:
            user_message["name"] = user_name
        self.messages.append(user_message)

        if stream:
            return str(self.get_response_stream())
        else:
            return str(self.get_response())

    def save_messages(self, file_path: str) -> None:
        import json

        with open(file_path, "w", encoding="utf-8") as f:
            json.dump(self.messages, f, ensure_ascii=False, indent=2)

    def load_messages(self, file_path: str) -> None:
        import json

        with open(file_path, "r", encoding="utf-8") as f:
            self.messages = json.load(f)

    def copy(self) -> Self:
        """深拷贝，用于多线程安全"""
        _agent = Agent.__new__(self.__class__)
        _agent._tools = self._tools
        _agent._client = self._client
        _agent._api_key = self._api_key
        _agent._base_url = self._base_url

        _agent.model = self.model
        _agent.instructions = self.instructions

        _agent.frequency_penalty = self.frequency_penalty
        _agent.temperature = self.temperature
        _agent.top_p = self.top_p
        _agent.max_tokens = self.max_tokens
        _agent.max_completion_tokens = self.max_completion_tokens
        _agent.message_expire_time = self.message_expire_time

        _agent.messages = deepcopy(self.messages)
        _agent.response_handlers = self.response_handlers.copy()
        _agent.stream_chunk_handlers = self.stream_chunk_handlers.copy()
        _agent.tool_call_handlers = self.tool_call_handlers.copy()
        return _agent

    @contextmanager
    def safe_modify(self, merge_messages: bool = True) -> Generator[Self, None, None]:
        """
        线程安全地更改messages，会在一轮对话结束后再追加更新的消息，并且不会改变其他属性。
        注意：过期的消息仍然会被清理
        """
        if self.message_expire_time:
            self.clear_msg_by_time(self.message_expire_time)
        _agent: Self = self.copy()
        yield _agent
        if merge_messages:
            added_messages: list[MessageParam] = _agent.messages
            for message in self.messages:
                if not message in added_messages:
                    break
                added_messages.remove(message)
            self.messages.extend(added_messages)

    def clear_msg(self) -> None:
        """清空消息，仅保留系统消息"""
        self.messages = [self.messages[0]]

    def clear_msg_by_time(self, expire_time: int) -> None:
        """
        清空消息，仅保留系统消息和最近若干秒内的消息

        :param expire_time: 过期时间，单位为秒
        """
        import time

        for message in self.messages[1:]:
            if int(time.time()) - message.get("time", 0) > expire_time:
                self.messages.remove(message)

    def add_response_handler(self, handler: Callable[[AssistantMessageParam], None]) -> None:
        """添加一个响应处理函数，当收到模型响应时，会调用该函数。函数的第一个（且是唯一一个）参数应当是模型输出的消息，以字典形式返回"""
        self.response_handlers.append(handler)

    def remove_response_handler(self, handler: Callable[[AssistantMessageParam], None]) -> None:
        self.response_handlers.remove(handler)

    def add_stream_chunk_handler(self, handler: Callable[[str], None]) -> None:
        """添加一个流式响应处理函数，当收到模型响应时，会调用该函数。只有在stream=True时，才会生效。函数的第一个（且是唯一一个）参数应当是模型输出的单个词语，以字符串形式返回"""
        self.stream_chunk_handlers.append(handler)

    def remove_stream_chunk_handler(self, handler: Callable[[str], None]):
        self.stream_chunk_handlers.remove(handler)

    def add_tool_call_handler(self, handler: Callable[[ToolCallParam], None]) -> None:
        """添加一个工具调用处理函数，当收到模型调用请求时，会调用该函数。函数的第一个（且是唯一一个）参数应当是模型的工具调用，以字典形式返回"""
        self.tool_call_handlers.append(handler)

    def remove_tool_call_handler(self, handler: Callable[[ToolCallParam], None]) -> None:
        self.tool_call_handlers.remove(handler)

    def add_reasoning_handler(self, handler: Callable[[str], None]) -> None:
        """添加一个推理处理函数，当收到模型推理请求时，会调用该函数。函数的第一个（且是唯一一个）参数应当是模型的推理请求，以字符串形式返回"""
        self.reasoning_handlers.append(handler)

    def remove_reasoning_handler(self, handler: Callable[[str], None]) -> None:
        self.reasoning_handlers.remove(handler)

    def add_stream_reasoning_handler(self, handler: Callable[[str], None]) -> None:
        """添加一个流式推理处理函数，当收到模型推理请求时，会调用该函数。只有在stream=True时，才会生效。函数的第一个（且是唯一一个）参数应当是模型的推理请求，以字符串形式返回"""
        self.stream_reasoning_handlers.append(handler)

    def remove_stream_reasoning_handler(self, handler: Callable[[str], None]) -> None:
        self.stream_reasoning_handlers.remove(handler)



from .base_tool import Tool
from abc import ABC
from typing import Any
from collections.abc import Callable, Awaitable
from typing import TypeAlias
from volcenginesdkarkruntime.types.chat import ChatCompletionToolParam
import inspect

FunctionParameters: TypeAlias = dict[str, object]


class BaseFunctionTool(Tool, ABC):

    def __init__(self, func: Callable[..., str | Awaitable[str]]) -> None:
        if isinstance(func, classmethod):
            raise TypeError(
                "FunctionTool cannot be used as a class method, please use @staticmethod to decorate the function."
            )
        elif not callable(func):
            raise TypeError("FunctionTool must be a callable object.")

        sig = inspect.signature(func)
        props: dict[str, Any] = {}
        required: list[str] = []

        for param_name, param in sig.parameters.items():
            param_type = "string"

            # 获取参数类型注解
            if param.annotation != inspect.Parameter.empty:

                # 处理基本类型
                if param.annotation == str:
                    param_type = "string"
                elif param.annotation == int or param.annotation == float:
                    param_type = "number"
                elif param.annotation == bool:
                    param_type = "boolean"

                # 处理嵌套类型
                elif hasattr(param.annotation, "__origin__"):
                    origin = param.annotation.__origin__

                    if origin == list:
                        param_type = "array"
                        if hasattr(param.annotation, "__args__"):
                            item_type = param.annotation.__args__[0]
                            item_schema = {}

                            # 处理列表中元素的类型
                            if item_type == str:
                                item_schema = {"type": "string"}
                            elif item_type == int or item_type == float:
                                item_schema = {"type": "number"}
                            elif item_type == bool:
                                item_schema = {"type": "boolean"}
                            elif (
                                item_type == dict
                                or getattr(item_type, "__origin__", None) == dict
                            ):
                                item_schema = {"type": "object"}

                            # 如果项类型已确定，添加 items 字段
                            if item_schema:
                                props[param_name] = {
                                    "type": "array",
                                    "description": f"{param_name} parameter",
                                    "items": item_schema,
                                }
                                continue

                else:  # 其他类型默认为 object
                    param_type = "object"

            props[param_name] = {
                "type": param_type,
                "description": f"{param_name} parameter",
            }

            # 如果有必需的参数，添加到 required 字段
            if param.default == inspect.Parameter.empty:
                required.append(param_name)

        parameters: FunctionParameters = {
            "type": "object",
            "properties": props,
        }

        if required:
            parameters["required"] = required

        self.parameters = parameters
        self.description: str = func.__doc__.strip() if func.__doc__ else ""
        self.name = func.__name__
        self.__annotations__ = func.__annotations__
        self.__signature__ = inspect.signature(func)
        self.__module__ = func.__module__
        self.__doc__ = func.__doc__
        self.__name__ = func.__name__
        self.__qualname__ = func.__qualname__

    def __repr__(self) -> str:
        return f"FunctionTool(name={self.name}, description={self.description}, parameters={self.parameters})"

    def to_dict(self) -> ChatCompletionToolParam:
        return {
            "type": "function",
            "function": {
                "name": self.name,
                "description": self.description,
                "parameters": self.parameters,
            },
        }


class FunctionTool(BaseFunctionTool):
    def __init__(self, func: Callable[..., str]) -> None:
        super().__init__(func)
        self._func: Callable[..., str] = func

    def __call__(self, *args, **kwds) -> str:
        return self._func(*args, **kwds)

    def __get__(self, obj, objtype) -> "FunctionTool":
        @FunctionTool
        def tool(*args, **kwds) -> str:
            return self._func(obj, *args, **kwds)

        tool.name = self.name
        tool.parameters = self.parameters
        tool.parameters["properties"] = dict(
            list(self.parameters["properties"].items())[1:]  # type: ignore
        )
        tool.parameters["required"] = self.parameters["required"][1:]  # type: ignore
        tool.description = self.description
        return tool


class FoldableFunctionTool(FunctionTool):
    foldable = True

    def __init__(self, func: Callable[..., str]):
        super().__init__(func)
        self.description += (
            "This tool is foldable, which means the result of this tool "
            "will be hidden when a new round of conversation starts."
        )


class AsyncFunctionTool(BaseFunctionTool):
    def __init__(self, func: Callable[..., Awaitable[str]]):
        super().__init__(func)
        self._func: Callable[..., Awaitable[str]] = func

    def __call__(self, *args, **kwargs) -> Awaitable[str]:
        return self._func(*args, **kwargs)

    def __get__(self, obj, objtype) -> "AsyncFunctionTool":
        @AsyncFunctionTool
        async def tool(*args, **kwds) -> str:
            return await self._func(obj, *args, **kwds)

        tool.name = self.name
        tool.parameters = self.parameters
        tool.parameters["properties"] = dict(
            list(self.parameters["properties"].items())[1:]  # type: ignore
        )
        tool.parameters["required"] = self.parameters["required"][1:]  # type: ignore
        tool.description = self.description
        return tool


class FoldableAsyncFunctionTool(AsyncFunctionTool):
    foldable = True

    def __init__(self, func: Callable[..., Awaitable[str]]):
        super().__init__(func)
        self.description += (
            "This tool is foldable, which means the result of this tool "
            "will be hidden when a new round of conversation starts."
        )



from abc import ABC, abstractmethod
from typing import TypeAlias
from collections.abc import Awaitable
from volcenginesdkarkruntime.types.chat import ChatCompletionToolParam

FunctionParameters: TypeAlias = dict[str, object]


class Tool(ABC):
    foldable = False
    name: str = "undefined"
    parameters: FunctionParameters = {}
    description: str = ""

    @abstractmethod
    def __call__(self, *args, **kwargs) -> str | Awaitable[str]:
        pass

    @abstractmethod
    def __repr__(self) -> str:
        pass

    @abstractmethod
    def to_dict(self) -> ChatCompletionToolParam:
        pass
